@@ -1,23 +1,23 @@
 # -*- coding: utf-8 -*-
 
-#-----------------------------------------------------------------------#
-# This file is part of Py-gtktree.                                      #
-#                                                                       #
-# Copyright (C) 2006, 2007, 2008, 2009 Paul Pogonyshev.                 #
-#                                                                       #
-# This program is free software: you can redistribute it and/or modify  #
-# it under the terms of the GNU General Public License as published by  #
-# the Free Software Foundation, either version 3 of the License, or (at #
-# your option) any later version.                                       #
-#                                                                       #
-# This program is distributed in the hope that it will be useful, but   #
-# WITHOUT ANY WARRANTY; without even the implied warranty of            #
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     #
-# General Public License for more details.                              #
-#                                                                       #
-# You should have received a copy of the GNU General Public License     #
-# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
-#-----------------------------------------------------------------------#
+#--------------------------------------------------------------------------#
+# This file is part of Py-gtktree.                                         #
+#                                                                          #
+# Copyright (C) 2006, 2007, 2008, 2009 Paul Pogonyshev.                    #
+#                                                                          #
+# This program is free software: you can redistribute it and/or modify it  #
+# under the terms of the GNU Lesser General Public License as published by #
+# the Free Software Foundation, either version 3 of the License, or (at    #
+# your option) any later version.                                          #
+#                                                                          #
+# This program is distributed in the hope that it will be useful, but      #
+# WITHOUT ANY WARRANTY; without even the implied warranty of               #
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser #
+# General Public License for more details.                                 #
+#                                                                          #
+# You should have received a copy of the GNU Lesser General Public License #
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.    #
+#--------------------------------------------------------------------------#
 
 
 import gtk
@@ -25,7 +25,8 @@ import operator
 
 
 __all__ = ('RowObjectTreeModel',
-           'RowObjectListStore')
+           'RowObjectListStore',
+           'TreeNode', 'DefaultTreeNode', 'LazyDefaultTreeNode', 'RowObjectTreeStore')
 
 
 
@@ -65,6 +66,10 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
     def columns (self):
         return self.__columns
 
+    @property
+    def node_based (self):
+        return False
+
 
     def get_cell (self, row, attribute):
         return self.get_value (self.get_iter (row), self.columns (attribute))
@@ -83,9 +88,18 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
     def get_row_object (self, iter):
         return self._do_get_row_object (self.get_user_data (iter))
 
+    def get_node (self, iter):
+        return self._do_get_node (self.get_user_data (iter))
+
     def _do_get_row_object (self, row):
         raise NotImplemented
 
+    def _do_get_node (self, row):
+        if self.node_based:
+            raise NotImplemented
+        else:
+            raise RuntimeError ("this model is not node-based")
+
     def _do_get_iter (self, row):
         raise NotImplemented
 
@@ -163,7 +177,10 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
 
         if self.is_accepting_drags_from (source_model):
             source_iter = source_model.get_iter (source_path)
-            return source_model, source_path, source_model.get_row_object (source_iter)
+            if not self.node_based:
+                return source_model, source_path, source_model.get_row_object (source_iter)
+            else:
+                return source_model, source_path, source_model.get_node (source_iter)
         else:
             return None, None, None
 
@@ -182,7 +199,7 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
 
 class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDest):
 
-    __gtype_name__ = 'RowObjectTreeModel'
+    __gtype_name__ = 'RowObjectListStore'
 
     def __init__(self, columns, values = None):
         if values is None:
@@ -617,11 +634,164 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
         else:
             return False
 
+    def do_row_drop_possible (self, dest_path, selection_data):
+        return True
+
+
+
+from gtktree._impl.tree_node import TreeNode, DefaultTreeNode, LazyDefaultTreeNode
+
+TreeNode           .__module__ = __name__
+DefaultTreeNode    .__module__ = __name__
+LazyDefaultTreeNode.__module__ = __name__
+
+
+# IMPLEMENTATION NOTES
+#
+# Iterator user data are the nodes themselves.  Because of this, we can set
+# 'leak_references' to False.  Also, by definition, iterators survive all tree store
+# modifications.
+
+class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDest):
+
+    __gtype_name__ = 'RowObjectTreeStore'
+
+    def __init__(self, columns, top_level_nodes = None):
+        self.__root            = _RootTreeNode (None, top_level_nodes)
+        self.__root.row_object = self
+
+        RowObjectTreeModel.__init__(self, columns)
+
+        self.props.leak_references = False
+
+
+    @property
+    def node_based (self):
+        return True
+
+
+    @property
+    def root (self):
+        return self.__root
+
+
+    def __len__(self):
+        # Default value comes from gtk.GenericTreeModel.
+        raise TypeError ("object of type '%s' has no len()" % type (self).__name__)
+
+
+    def __contains__(self, row_object):
+        # FIXME
+        return False
+
+    def __iter__(self):
+        # Default value comes from gtk.GenericTreeModel.
+        raise TypeError ("'%s' object is not iterable" % type (self).__name__)
+
+
+    def _create_index_error (self, index):
+        return IndexError ("invalid row index %s: "
+                           "only integers in range %d..%d (inclusive) and gtk.TreeIter are valid)"
+                           % (index, -len (self), len (self) - 1))
+
+
+    def on_get_flags (self):
+        return gtk.TREE_MODEL_ITERS_PERSIST
+
+
+    def _do_get_row_object (self, row):
+        return row.row_object
+
+    def _do_get_node (self, row):
+        return row
+
+    def _do_get_iter (self, row):
+        if isinstance (row, gtk.TreeIter):
+            return row
+        else:
+            return self.create_tree_iter (row)
+
+
+    def on_get_iter (self, path):
+        node = self.root
+        for index in path:
+            node = node.get_child_node (index)
+        return node
+
+    def on_get_path (self, row):
+        return row.compute_path ()
+
+
+    def on_iter_parent (self, row):
+        parent = row.parent_node
+        return parent if parent is not self.root else None
+
+    def on_iter_next (self, row):
+        return row.next_node
+
+    def on_iter_has_child (self, row):
+        return row.has_child_nodes
+
+    def on_iter_children (self, row):
+        return row.first_child_node
+
+    def on_iter_n_children (self, row):
+        return (row if row is not None else self.root).num_child_nodes
+
+    def on_iter_nth_child (self, parent, n):
+        if parent is not None:
+            return parent.get_child_node (n)
+        else:
+            return self.root.get_child_node (n)
+
+
+    # Implementing gtk.TreeDragSource.
+
+    def do_row_draggable (self, path):
+        return True
+
+    def do_drag_data_delete (self, path):
+        node = self.get_node (self.get_iter (path))
+        node.parent_node.delete_child_node (node)
+        return True
+
+    def do_drag_data_get (self, path, selection_data):
+        return selection_data.tree_set_row_drag_data (self, path)
+
+
+    # Implementing gtk.TreeDragDest.
+
+    def do_drag_data_received (self, dest_path, selection_data):
+        source_model, source_path, node = self._get_drag_data (selection_data)
+
+        if source_model is not None:
+            node = node.hierarchic_copy ()
+
+            # This convoluted logic is meant to cope with drag-to-the-very-end situations.
+            if dest_path and dest_path[-1] > 0:
+                dest_path      = list (dest_path)
+                dest_path[-1] -= 1
+
+                dest_node = self.get_node (self.get_iter (tuple (dest_path)))
+                dest_node.parent_node.insert_child_node (node, dest_node.next_node)
+            else:
+                dest_node_parent = self.get_node (self.get_iter (dest_path)).parent_node
+                dest_node_parent.insert_child_node (node, dest_node_parent.first_child_node)
+
+            return True
+        else:
+            return False
 
     def do_row_drop_possible (self, dest_path, selection_data):
         return True
 
 
+class _RootTreeNode (DefaultTreeNode):
+
+    def _do_get_tree_model (self):
+        return self.row_object
+
+
 
 # Local variables:
 # mode: python
