@@ -1,23 +1,23 @@
 # -*- coding: utf-8 -*-
 
-#-----------------------------------------------------------------------#
-# This file is part of Py-gtktree.                                      #
-#                                                                       #
-# Copyright (C) 2008, 2009 Paul Pogonyshev.                             #
-#                                                                       #
-# This program is free software: you can redistribute it and/or modify  #
-# it under the terms of the GNU General Public License as published by  #
-# the Free Software Foundation, either version 3 of the License, or (at #
-# your option) any later version.                                       #
-#                                                                       #
-# This program is distributed in the hope that it will be useful, but   #
-# WITHOUT ANY WARRANTY; without even the implied warranty of            #
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     #
-# General Public License for more details.                              #
-#                                                                       #
-# You should have received a copy of the GNU General Public License     #
-# along with this program.  If not, see <http://www.gnu.org/licenses/>. #
-#-----------------------------------------------------------------------#
+#--------------------------------------------------------------------------#
+# This file is part of Py-gtktree.                                         #
+#                                                                          #
+# Copyright (C) 2008, 2009 Paul Pogonyshev.                                #
+#                                                                          #
+# This program is free software: you can redistribute it and/or modify it  #
+# under the terms of the GNU Lesser General Public License as published by #
+# the Free Software Foundation, either version 3 of the License, or (at    #
+# your option) any later version.                                          #
+#                                                                          #
+# This program is distributed in the hope that it will be useful, but      #
+# WITHOUT ANY WARRANTY; without even the implied warranty of               #
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser #
+# General Public License for more details.                                 #
+#                                                                          #
+# You should have received a copy of the GNU Lesser General Public License #
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.    #
+#--------------------------------------------------------------------------#
 
 
 if __name__ == '__main__':
@@ -27,32 +27,11 @@ if __name__ == '__main__':
 
 import unittest
 
+from test._common  import Foo, PopAllList
 from gtktree.model import RowObjectListStore
 
 
 
-class Foo (object):
-
-    __slots__ = ('foo', 'bar')
-
-    def __init__(self, foo, bar = None):
-        self.foo = foo
-        self.bar = bar
-
-    def __eq__(self, other):
-        return self.foo == other.foo and self.bar == other.bar
-
-    def __ne__(self, other):
-        return self.foo != other.foo or self.bar != other.bar
-
-    def __repr__(self):
-        if self.bar is not None:
-            return '%s(%s, %s)' % (self.__class__.__name__, self.foo, self.bar)
-        else:
-            return '%s(%s)' % (self.__class__.__name__, self.foo)
-
-
-
 # Some tests assume current order of row changing, insertion or removal.  Such tests will
 # fail if this order is changed for whatever reason and those failures won't necessary
 # indicate real bugs.  In that case, you will need to check and possibly alter these
@@ -76,12 +55,12 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_create_with_values (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
         self.assertEqual (len (store), 2)
 
 
     def test_get_item (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         self.assertEqual (store[0], Foo (1))
         self.assertEqual (store[1], Foo (2, 'abc'))
@@ -89,7 +68,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_get_value (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         row = store.get_iter_first ()
         self.assertEqual (store.get_value (row, 0), 1)
@@ -101,7 +80,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_get_cell (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         self.assertEqual (store.get_cell (0, 'foo'), 1)
         self.assertEqual (store.get_cell (0, 'bar'), None)
@@ -110,7 +89,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_set_item (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         store[0] = Foo (10)
         self.assertEqual (store[0], Foo (10))
@@ -133,7 +112,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_set_slice (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         store[1:1] = (Foo (10), Foo (20))
         self.assertEqual (len (store), 4)
@@ -159,7 +138,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_set_extended_slice (self):
-        store, emissions = self._create_foo_list_store (Foo (index) for index in range (10))
+        store, emissions = _create_foo_list_store (Foo (index) for index in range (10))
 
         store[1:8:3] = Foo (-1), Foo (-2), Foo (-3)
         self.assertEqual (len (store), 10)
@@ -177,7 +156,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_append (self):
-        store, emissions = self._create_foo_list_store ([])
+        store, emissions = _create_foo_list_store ([])
 
         store.append (Foo (10))
         self.assertEqual (len (store), 1)
@@ -192,7 +171,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_prepend (self):
-        store, emissions = self._create_foo_list_store ([])
+        store, emissions = _create_foo_list_store ([])
 
         store.prepend (Foo (10))
         self.assertEqual (len (store), 1)
@@ -207,7 +186,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_insert (self):
-        store, emissions = self._create_foo_list_store ([])
+        store, emissions = _create_foo_list_store ([])
 
         store.insert (0, Foo (10))
         self.assertEqual (len (store), 1)
@@ -229,7 +208,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_insert_after (self):
-        store, emissions = self._create_foo_list_store ([])
+        store, emissions = _create_foo_list_store ([])
 
         store.insert_after (0, Foo (10))
         self.assertEqual (len (store), 1)
@@ -251,7 +230,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_insert_negative_position (self):
-        store, emissions = self._create_foo_list_store ([Foo (0), Foo (1), Foo (2)])
+        store, emissions = _create_foo_list_store ([Foo (0), Foo (1), Foo (2)])
 
         store.insert (-2, Foo (10))
         self.assertEqual (len (store), 4)
@@ -260,7 +239,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_insert_out_of_bounds (self):
-        store, emissions = self._create_foo_list_store ([Foo (0), Foo (1), Foo (2)])
+        store, emissions = _create_foo_list_store ([Foo (0), Foo (1), Foo (2)])
 
         # As with Python lists, we expect out-of-bounds positions be silently clamped.
         store.insert (100, Foo (100))
@@ -275,7 +254,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_extend (self):
-        store, emissions = self._create_foo_list_store ([Foo (0)])
+        store, emissions = _create_foo_list_store ([Foo (0)])
 
         store.extend ([Foo (10), Foo (20), Foo (30)])
         self.assertEqual (len (store), 4)
@@ -289,7 +268,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_remove (self):
-        store, emissions = self._create_foo_list_store ([Foo (1), Foo (2), Foo (3), Foo (4)])
+        store, emissions = _create_foo_list_store ([Foo (1), Foo (2), Foo (3), Foo (4)])
 
         store.remove (Foo (3))
         self.assertEqual (len (store), 3)
@@ -298,7 +277,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
         self.assertEqual (store[2], Foo (4))
         self.assertEqual (emissions.pop_all (), [('deleted', (2,))])
 
-        store, emissions = self._create_foo_list_store ([Foo (1), Foo (2), Foo (3), Foo (4),
+        store, emissions = _create_foo_list_store ([Foo (1), Foo (2), Foo (3), Foo (4),
                                                          Foo (3), Foo (2), Foo (1)])
         store.remove (Foo (3))
         self.assertEqual (len (store), 6)
@@ -314,7 +293,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_pop (self):
-        store, emissions = self._create_foo_list_store ([Foo (1), Foo (2), Foo (3), Foo (4)])
+        store, emissions = _create_foo_list_store ([Foo (1), Foo (2), Foo (3), Foo (4)])
 
         popped = store.pop ()
         self.assertEqual (len (store), 3)
@@ -333,7 +312,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_clear (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
         self.assert_(len (store) == 2)
 
         store.clear ()
@@ -343,7 +322,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_reverse (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         contents = store[:]
 
@@ -356,34 +335,34 @@ class RowObjectListStoreTestCase (unittest.TestCase):
     # Note: we don't use Foo() here to simplify things.  We don't access columns anyway,
     # so this doesn't matter.
     def test_sort (self):
-        store, emissions = self._create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
+        store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort ()
         self.assertEqual (store[:], ['A', 'C', 'D', 'b', 'e'])
         self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 4, 3, 0, 2])])
 
-        store, emissions = self._create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
+        store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (key = str.lower)
         self.assertEqual (store[:], ['A', 'b', 'C', 'D', 'e'])
         self.assertEqual (emissions.pop_all (), [('reordered', '?' or [2, 4, 1, 0, 3])])
 
-        store, emissions = self._create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
+        store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (cmp = lambda a, b: -cmp (a, b))
         self.assertEqual (store[:], ['e', 'b', 'D', 'C', 'A'])
         self.assertEqual (emissions.pop_all (), [('reordered', '?' or [2, 0, 3, 4, 1])])
 
-        store, emissions = self._create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
+        store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (cmp = lambda a, b: -cmp (a, b), key = str.lower)
         self.assertEqual (store[:], ['e', 'D', 'C', 'b', 'A'])
         self.assertEqual (emissions.pop_all (), [('reordered', '?' or [3, 0, 1, 4, 2])])
 
-        store, emissions = self._create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
+        store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (reverse = True)
         self.assertEqual (store[:], ['e', 'b', 'D', 'C', 'A'])
         self.assertEqual (emissions.pop_all (), [('reordered', '?' or [2, 0, 3, 4, 1])])
 
 
     def test_swap (self):
-        store, emissions = self._create_foo_list_store ([Foo (0), Foo (1), Foo (2), Foo (3)])
+        store, emissions = _create_foo_list_store ([Foo (0), Foo (1), Foo (2), Foo (3)])
 
         store.swap (1, 3)
         self.assertEqual (store[1], Foo (3))
@@ -434,7 +413,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def test_note_changes (self):
-        store, emissions = self._create_foo_list_store ()
+        store, emissions = _create_foo_list_store ()
 
         self.assertEqual (store.note_changes (1), 1)
         self.assertEqual (emissions.pop_all (), [('changed', (1,), 1)])
@@ -445,7 +424,7 @@ class RowObjectListStoreTestCase (unittest.TestCase):
 
 
     def _do_test_move (self, mover, expected_result):
-        store, emissions = self._create_foo_list_store ([0, 1, 2, 3, 4])
+        store, emissions = _create_foo_list_store ([0, 1, 2, 3, 4])
 
         if expected_result is None:
             mover (store)
@@ -461,42 +440,36 @@ class RowObjectListStoreTestCase (unittest.TestCase):
             self.assertRaises (expected_result, lambda: mover (store))
 
 
-    def _create_foo_list_store (self, values = None):
-        if values is None:
-            values = [Foo (1), Foo (2, 'abc')]
 
-        store = RowObjectListStore ([(int, 'foo'),
+def _create_foo_list_store (values = None):
+    if values is None:
+        values = [Foo (1), Foo (2, 'abc')]
+
+    store     = RowObjectListStore ([(int, 'foo'),
                                      (str, 'bar')],
                                     values)
+    emissions = PopAllList ()
 
-        class PopAllList (list):
-            def pop_all (self):
-                result = self[:]
-                del self[:]
-                return result
+    store.connect ('row-changed',
+                   lambda model, path, iter:
+                       emissions.append (('changed', path, model.get_row_index (iter))))
+    store.connect ('row-deleted',
+                   lambda model, path:
+                       emissions.append (('deleted', path)))
+    store.connect ('row-inserted',
+                   lambda model, path, iter:
+                       emissions.append (('inserted', path, model.get_row_index (iter))))
 
-        emissions = PopAllList ()
+    def reordered (model, path, iter, new_order):
+        if not path and iter is None:
+            # 'new_order' is not readable in Python (it is gobject.GPointer)...
+            emissions.append (('reordered', '?'))
+        else:
+            emissions.append (('reordered', path, iter, '?'))
 
-        store.connect ('row-changed',
-                       lambda model, path, iter:
-                           emissions.append (('changed', path, model.get_row_index (iter))))
-        store.connect ('row-deleted',
-                       lambda model, path:
-                           emissions.append (('deleted', path)))
-        store.connect ('row-inserted',
-                       lambda model, path, iter:
-                           emissions.append (('inserted', path, model.get_row_index (iter))))
+    store.connect ('rows-reordered', reordered)
 
-        def reordered (model, path, iter, new_order):
-            if not path and iter is None:
-                # 'new_order' is not readable in Python (it is gobject.GPointer)...
-                emissions.append (('reordered', '?'))
-            else:
-                emissions.append (('reordered', path, iter, '?'))
-
-        store.connect ('rows-reordered', reordered)
-
-        return store, emissions
+    return store, emissions
 
 
 
